function [H] = homografia( points1, points2)
%homografia: calcula la homografa entre puntos de dos planos proyectivos.
%  Requiere un mnimo de 4 puntos. 
%  Parmetros:
%     points1: array bidimensional  con coordenadas 2D   
%            o 3D(en este caso se descarta la tercera coordenada)
%           Una columna por cada punto
%     points2: array bidimensional con coordenadas 2D   
%            o 3D(en este caso se descarta la tercera coordenada)
%           Una columna por cada punto
%
%   Valores devueltos:
%     H: matriz 3x3 tal que   r*[u; v; 1] = H *[X; Y; 1]
%           u= points1(1,i)     X= points2(1,i)
%           v= points1(2,i)     Y= points2(2,i)

% inicializa variables
H = zeros(3,3);

% Comprueba dimensiones
[f1, c1] = size(points1);
[f2, c2] = size(points2);

% Comprueba que al menos hay 4 puntos y son compatibles ambos arrays (mismo
% nmero de puntos)
if c1<4 || c2<4 || c1~=c2 || f1<2 || f2<2
    return;
end

% nmero puntos a procesar
num_puntos = size(points1,2);

%-----------------------------------------------------
% CDIGO OPCIONAL: normalizar coordenadas Obtener T1,T2
% hvec1 = T1*hvec1;  hvec2 = T2*hvec2;  % puntos  -> vectores columna
%-----------------------------------------------------
T1= eye(3); % transformacin coordenas imagen (points1)
T2= eye(3); % transformacin coordenas plano (points2)



%-----------------------------------------------------
% aplica la transformacin en coordenadas homogeneas (por columnas)
% y extrae las nuevas coordandas euclideas
hvec = [points1(1:2,:); ones(1,size(points1,2))]; % pasa a coordenadas homogeneas
hvec = T1*hvec;
for i=1: size(points1,2)
    points1(1:2,i) = hvec(1:2,i)./hvec(3,i);  % extrae coordenadas euclideas
end

hvec = [points2(1:2,:); ones(1,size(points2,2))];   % pasa a coordenadas homogeneas
hvec = T2*hvec;
for i=1: size(points2,2)
    points2(1:2,i) = hvec(1:2,i)./hvec(3,i);  % extrae coordenadas euclideas
end


%-----------------------------------------------
% Aadir el cdigo de clculo de H  -> [r*u; r*v; r] = H *[X; Y; 1]
%           u= points1(1,i)     X= points2(1,i)
%           v= points1(2,i)     Y= points2(2,i)
%-----------------------------------------------
A=[];
for i= 1:num_puntos
    u = points1(1,i);  X = points2(1,i);
    v = points1(2,i);  Y = points2(2,i);  
    
    %----------------------------------------------------------
    % Aadir el cdigo para crear el sistema de ecuaciones A*h=0 
    %----------------------------------------------------------

    
    
    %----------------------------------------------------------
end

%---------------------------------------------------------------
% Aadir el cdigo para resolver el sistema de ecuaciones A*h=0
% Mostrar la condicin  del sistema de ecuaciones = VS(1)/VS(8)
%---------------------------------------------------------------
h=[];


%-----------------------------------------------------------------
% Aadir el cdigo para obtener la matriz H a partir del vector h 
%-----------------------------------------------------------------




%----------------------------------------------------------

%--------------------------------------------------------
% Transformacin des-normalizacin
% H = inv(T1)*H*T2
%--------------------------------------------------------
H = inv(T1)*H*T2;

if H(3,3)~=0
    H = H/H(3,3);   % Normaliza H
end

%----------------------------------------------------------
end % fin de la funcin