% Practica Estimacin de POSE: posicin y orientacin a partir de marcas y
% cmara calibrada.
% Escenario 1: una sola marca (id==1) en imagen capturada de la cmara
close all;  %cierra todas las figuras

% Carga datos de la cmara (obtenida con la toolbox de calibracin)
% Contiene: fc=[fx; fy] cc=[Cx; Cy]  Distorsion: kc=[k1; k2; p1; p2; k3]
%           fc_error    cc_error                 kc_error
%           Dimensiones imagen: nx (columnas), ny (filas) 
%
%calib_Trust;
%calib_LogitechC300;
calib_LogitechWP9000;

% construye matriz de parmetros intrnsecos sin distorsin 
A = [fc(1), 0 , cc(1); 0, fc(2), cc(2); 0 0 1];

disp('Matriz Cmara (A):'); disp(A);

% Tamao de la imagen (para la normalizacin)
Nf=ny; Nc=nx;   

% Carga datos de la marca (posicin de las esquinas). Contiene: 
%   marker_ids: (array of integers )
%   marker_3Dcorners: 3D coordinates stored as a thre-dimensional array (j,k,i): 
%             j: index thre-dimensional coordinates (X,Y,Z) 
%             k: index four corners of the marker
%             i: index of the marker_ids array 
marker_data;

% Detecta marcas con una cmara USB (pulsar la tecla ESC para devolver las
% marcas detectadas). Valores devueltos:
%     id: es un vector con los identificadores de las marcas detectadas
%     points: es una matriz de tres dimensiones points(j,k,i): 
%             j: ndice de las dos coordenadas (f,c)
%             k: ndice de las 4 esquinas del marcador 
%             i: es el ndice  en el vector de ids 
%     im: imagen procesada
[id, points, im] = markers('Image1.jpg');  % procesa imagen de ejemplo (usar fichero de calibracion calib_LogitechWP9000) 
%[id, points, im] = markers;

% Muestra una figura con la imagen y las esquinas de las marcas detectadas
showmarkers(id, points, im);

% Busca en el array marker_3Dcorners los datos de las marcas detectadas
% El orden en el que se detectan las marcas no tiene por qu coincidir con
% el orden del fichero de datos
marker_3Dcorners_detected = [];
for i = 1:size(id,2)
    % busca id(i) en el array marker_ids
    k = find(marker_ids == id(i));
    % copia los datos de la marca
    if ~isempty(k)
        marker_3Dcorners_detected(:,:,i) = marker_3Dcorners(:,:,k);
    end
end

if size(marker_3Dcorners_detected)==0
    disp('No se han encontrado marcas detectadas coincidentes con el fichero de datos');
    disp(newline);
    return;
end

% Si es una array de tres dimensiones (lista de marcas) lo convierte a uno bidimensional
[f1, c1, i1] = size(points);
[f2, c2, i2] = size(marker_3Dcorners_detected);
if i1>1
    points = reshape(points, [f1, c1*i1]); 
end
if i2>1
    marker_3Dcorners_detected = reshape(marker_3Dcorners_detected, [f2, c2*i2]); 
end

%------------------------------------------------------
% CODIGO OPCIONAL: corregir distorsin en array points
%------------------------------------------------------


%-----------------------------------------------
% CODIGO OPCIONAL:  aadir ruido al array points 
%-----------------------------------------------


% Calcular la homografa con los puntos 2D de la imagen (points)  y los puntos 3D
% de la marca (marker_3Dcorners_detected)
%   [r*u; r*v; r] = H *[X; Y; 1]
%
%  NOTA: El alumno deber aadir el cdigo necesario en la funcin homografia.m
H = homografia(points, marker_3Dcorners_detected);

disp('Homografia:'); disp(H);

%----------------------------------------------------
% Aadir el cdigo de clculo de R,T a partir de H,A
% [r1 r2 t]=lamda*inv(A)[h1 h2 h3]
%  ||r1||=1    ||r2||=1   r3=r1xr2 
%----------------------------------------------------
% Inicializa Rotacin y traslacin
R = eye(3,3);
t = [0 0 1]';   % tz no puede ser cero





%--------------------------------------------------------
% CDIGO OPCIONAL: Dibujar sistema coordenadas 3D
%   m = A*[R t]*M
%--------------------------------------------------------
%axis3D(A, R, t);


%-----------------------------------------------
%Muestra Rotacin y traslacin
disp('Rotacin:'); disp(R);
disp('Traslacin:'); disp(t);

%--------------------------------------------------------
% Error de reproyeccin
%--------------------------------------------------------
Mh = [marker_3Dcorners_detected; ones(1,size(marker_3Dcorners_detected, 2))]; % Puntos 3D en coordenadas homogeneas
mh = A*[R t]*Mh;        % reproyecta puntos 3D
for i=1: size(mh,2)
    mh(:,i) = mh(:,i) ./ mh(3,i);     % normaliza por el factor de escala
end
m = mh(1:2, :);          % coordenadas 2D Euclideas

% error cuadrtico medio
error = sum(sum((points-m).*(points-m)))/size(m,2);
disp('Error Reproyecin: '); disp(error);

%-----------------------------------------------
% fin del programa
